import 'package:flutter/material.dart';

class EasyEverythingButton extends StatefulWidget {
  /// Button controller, now required
  final EasyEverythingButtonController controller;

  ///child label button
  final Widget child;

  ///background color button, default: primary color
  final Color? backgroundColor;

  ///hover color, default: hover color
  final Color? overlayColor;

  ///shadow color is long press, default shadowColor
  final Color? shadowColor;

  ///border radius, default 45/2
  final double borderRadius;

  /// default 45
  final double height;

  ///on tap button
  final Function onTap;

  ///duration hover effect
  final int durationHover;

  ///enable button = false is on tap not available
  final bool enableButton;

  const EasyEverythingButton({
    required this.controller,
    required this.child,
    this.height = 45,
    this.backgroundColor,
    this.overlayColor,
    this.shadowColor,
    this.borderRadius = 15,
    this.durationHover = 1500,
    this.enableButton = false,
    required this.onTap,
  });
  @override
  _EasyEverythingButtonState createState() => _EasyEverythingButtonState();
}

class _EasyEverythingButtonState extends State<EasyEverythingButton>
    with TickerProviderStateMixin {
  late Animation _bounceAnimation;
  late AnimationController _checkButtonControler;
  late AnimationController _borderController;
  late Animation _borderAnimation;
  @override
  void initState() {
    _borderController = AnimationController(
        duration: Duration(milliseconds: 1000), vsync: this);
    _borderAnimation = BorderRadiusTween(
            begin: BorderRadius.circular(widget.borderRadius),
            end: BorderRadius.circular(widget.height / 2))
        .animate(_borderController);

    _borderAnimation.addListener(() {
      print('val1 ${_borderAnimation.value}');
      setState(() {});
    });
    _checkButtonControler = AnimationController(
        duration: Duration(milliseconds: 1000), vsync: this);
    _bounceAnimation = Tween<double>(begin: 300, end: widget.height).animate(
        CurvedAnimation(
            parent: _checkButtonControler, curve: Curves.easeInOutCirc));
    _bounceAnimation.addListener(() {
      print('val ${_bounceAnimation.value}');
      setState(() {});
    });
    widget.controller.addListeners(_start, _stop, _success, _error, _reset);

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    final ThemeData theme = Theme.of(context);

    return ElevatedButton(
      style: ElevatedButton.styleFrom(
        minimumSize: Size(_bounceAnimation.value, widget.height),
        padding: EdgeInsets.all(0),
        shape: RoundedRectangleBorder(
          borderRadius: _borderAnimation.value,
        ),
      ),
      onPressed: () {
        _checkButtonControler.forward();
        _borderController.forward();
      },
      child: Text('w'),
    );
  }

  _start() {}

  _stop() {}

  _success() {}

  _error() {}

  _reset() {}
}

class EasyEverythingButtonController {
  late VoidCallback _startListener;
  late VoidCallback _stopListener;
  late VoidCallback _successListener;
  late VoidCallback _errorListener;
  late VoidCallback _resetListener;

  addListeners(
      VoidCallback startListener,
      VoidCallback stopListener,
      VoidCallback successListener,
      VoidCallback errorListener,
      VoidCallback resetListener) {
    _startListener = startListener;
    _stopListener = stopListener;
    _successListener = successListener;
    _errorListener = errorListener;
    _resetListener = resetListener;
  }

  /// Notify listeners to start the loading animation
  void start() {
    _startListener();
  }

  /// Notify listeners to start the stop animation
  void stop() {
    _stopListener();
  }

  /// Notify listeners to start the sucess animation
  void success() {
    _successListener();
  }

  /// Notify listeners to start the error animation
  void error() {
    _errorListener();
  }

  /// Notify listeners to start the reset animation
  void reset() {
    _resetListener();
  }
}
